library(data.table)
library(tidyhydat)
library(sf)
library(stars)

### SET DIRECTORIES ####
# Set root directory
wd_root <- "/Users/jordanfields/Documents/Graduate_School/Research/Arctic_Meander_Rate/Data_Analysis/October2023_Results/"
# Imports folder (store all import files here)
wd_imports <- paste0(wd_root,"GEE_Exports/")
# Exports folder (save all figures, tables here)
wd_exports <- paste0(wd_root,"R_Exports/")
wd_figures <- paste0(wd_exports, "Figures/")

# Get data.table of all stations with sampling metadata
hydat_station_info <- data.table(hy_stations())

# Get Mackenzie Stations
MackenzieSites = hydat_station_info[startsWith(STATION_NAME,'MACKENZIE RIVER')]

# Get data.table of unregulated stations. Start and end date are unreliable.
hydat_q_unregulated <- data.table(hy_stn_regulation())[REGULATED == FALSE]

# Merge tables
MackenzieSites = merge(MackenzieSites,hydat_q_unregulated[,c('STATION_NUMBER','REGULATED')],by='STATION_NUMBER',all.x=T)

# Download Data
MackenzieData = data.table()
for (hydat_site_no in MackenzieSites$STATION_NUMBER){
  download <- data.table(hy_monthly_flows(station_number=)) #hy_annual_stats #start_year = 1979-01-01, end_year = 2023-10-15 #hydat_site_no
  MackenzieData = rbind(MackenzieData,download)
}

# Select Mean Annual Flow
MackenzieData = MackenzieData[Parameter=='Flow' & Sum_stat =='MAX']
MackenzieData = MackenzieData[,-c(6,7)]
setnames(MackenzieData,'Value','Q_cms')

# Merge Site Information
MackenzieData = merge(MackenzieData,MackenzieSites[,c('STATION_NUMBER','STATION_NAME','LATITUDE','LONGITUDE')],BY='STATION_NUMBER')

write.csv(MackenzieData,paste0(wd_root, "MackenzieRiver_DischargeMAX.csv"))

#### NARR Extraction ####
# Import Basin
WSC = read_sf("C:\\Users\\cmeri\\OneDrive - Dartmouth College\\Research\\Arctic_Rivers\\Mackenzie\\Mackenzie River Basin.kml")
WSC = WSC[-2]
WSC = WSC[1,]

# Convert NetCDF to Raster
precip_raster = read_stars('C:\\Users\\cmeri\\OneDrive - Dartmouth College\\Research\\Runoff_Ratio\\ERA5\\Input\\NARR_Precip.nc',sub='apcp')
temperature_raster = read_stars('C:\\Users\\cmeri\\OneDrive - Dartmouth College\\Research\\Runoff_Ratio\\ERA5\\Input\\NARR_Temp.nc',sub='air')

# Reproject to WGS84
precip_raster = st_transform(precip_raster, '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs')
temperature_raster = st_transform(temperature_raster, '+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs')

# Calculate WSC Watershed Means
# Precipitation
WSC_Watersheds = as.data.frame(st_extract(precip_raster, at=WSC, fun=mean, na.rm=T))
Mackenzie_Precip = as.data.table(WSC_Watersheds)
Mackenzie_Precip = Mackenzie_Precip[,Precip_kg_m2:=gsub('[kg/m^2]','',apcp)]

# Temperature
WSC_Temp = as.data.frame(st_extract(temperature_raster, at=WSC, fun=mean, na.rm=TRUE))
Mackenzie_Temp = as.data.table(WSC_Temp)
Mackenzie_Temp = Mackenzie_Temp[,Temp_K:=gsub('[K]','',air)]

# Merge
Mackenzie_Climate = merge(Mackenzie_Precip[,c('time','Precip_kg_m2')],Mackenzie_Temp[,c('time','Temp_K')],by='time')
write.csv(Mackenzie_Climate,'C:\\Users\\cmeri\\OneDrive - Dartmouth College\\Research\\Arctic_Rivers\\Mackenzie\\Mackenzie_Climate.csv')
